import {registerExpressPaymentMethod} from '@woocommerce/blocks-registry';
import {getSetting} from '@woocommerce/settings';
import {__} from '@wordpress/i18n';
import {useLoadPayPalScript, usePaymentMethodNotices} from "../../hooks";
import {usePayPalOptions, usePayPalFundingSources, useProcessPayment} from "./hooks";
import './styles.scss';
import {useProcessPaymentFailure} from "../../hooks";
import {isAddressValid, isUserAdmin} from "../../utils";

const getData = (key) => {
    const data = getSetting(key);
    return (key, defaultValue = null) => {
        if (!data.hasOwnProperty(key)) {
            data[key] = defaultValue;
        }
        return data[key];
    };
}

const data = getData('ppcp_data');
const generalData = getData('ppcpGeneralData');

const isExpressEnabled = () => {
    const sections = ['paypalSections', 'payLaterSections', 'creditCardSections', 'venmoSections'];
    for (const section of sections) {
        if (data(section, []).includes('express_checkout')) {
            return true;
        }
    }
    return false;
}

const ExpressPaymentMethod = (props) => {
    return <PayPalPaymentMethod
        context={'express_checkout'}
        paymentMethodId='ppcp'
        {...props}/>;
}

const PaymentMethod = (props) => {
    const {addErrorNotice: onError} = usePaymentMethodNotices();
    return <PayPalPaymentMethod
        {...props}
        paymentMethodId='ppcp'
        isExpress={false}
        onError={onError}/>;
}

const PayPalPaymentMethod = (
    {
        isExpress = true,
        context,
        billing,
        shippingData,
        eventRegistration,
        emitResponse,
        onError,
        onClick,
        onClose,
        onSubmit,
        activePaymentMethod,
        paymentMethodId,
        ...props
    }) => {
    const queryParams = getSetting('paypalQueryParams');
    const vault = queryParams.vault === 'true';
    const {billingData} = billing;
    const {onPaymentProcessing, onCheckoutAfterProcessingWithError} = eventRegistration;
    const {responseTypes, noticeContexts} = emitResponse;

    const {paymentData, setPaymentData} = useProcessPayment({
        onSubmit,
        billingData,
        shippingData,
        onPaymentProcessing,
        responseTypes,
        activePaymentMethod,
        paymentMethodId
    });

    useProcessPaymentFailure({
        event: onCheckoutAfterProcessingWithError,
        responseTypes,
        messageContext: isExpress ? noticeContexts.EXPRESS_PAYMENTS : null,
        setPaymentData
    });

    const paypal = useLoadPayPalScript(queryParams);

    const {getOptions} = usePayPalOptions({
        isExpress,
        paypal,
        vault,
        intent: queryParams.intent,
        buttonStyles: data('buttons'),
        billing,
        shippingData,
        eventRegistration,
        setError: onError,
        setPaymentData,
        onClick,
        onClose
    });
    const sources = usePayPalFundingSources({
        data,
        paypal,
        context,
        vault
    });
    if (isExpress && paymentData) {
        const validAddress = isAddressValid(billingData);
        return (
            <div>
                {!validAddress &&
                <p className='wc-ppcp-payment__notice'>{__('Please fill out all required fields and click Place Order.', 'pymntpl-paypal-woocommerce')}</p>}
                {isUserAdmin() && !validAddress &&
                <p className='wc-ppcp-payment__notice'>{__('Admin notice: In order for customer billing address details to be auto-populated you ' +
                    'must contact PayPal Support and request this feature.', 'pymntpl-paypal-woocommerce')}</p>}
            </div>
        )
    }
    if (paypal && sources) {
        const Button = paypal.Buttons.driver("react", {React, ReactDOM});
        const BUTTONS = sources.map(source => {
            const options = getOptions(source);
            const button = paypal.Buttons(options);
            return button.isEligible() ? <Button key={source} {...options}/> : null;
        });
        return (
            <div className='wc-ppcp-paypal__buttons'>
                {BUTTONS}
            </div>
        );
    }
    return null;
}

const PaymentMethodLabel = ({components, title, icons, id}) => {
    if (!Array.isArray(icons)) {
        icons = [icons];
    }
    const {PaymentMethodLabel: Label, PaymentMethodIcons} = components;
    return (
        <div className={`wc-ppcp-blocks-payment-method__label ${id}`}>
            <Label text={title}/>
            <PaymentMethodIcons icons={icons}/>
        </div>
    )
};

if (isExpressEnabled()) {
    registerExpressPaymentMethod({
        name: 'ppcp',
        canMakePayment: () => true,
        content: <ExpressPaymentMethod/>,
        edit: <ExpressPaymentMethod/>,
        supports: {
            features: data('features')
        }
    });
}

/*registerPaymentMethod({
    name: 'ppcp',
    label: <PaymentMethodLabel
        id='ppcp'
        title={data('title')}
        icons={data('icons')}/>,
    ariaLabel: 'PayPal',
    canMakePayment: () => true,
    content: <PaymentMethod context={'checkout'}/>,
    edit: <PaymentMethod/>,
    supports: {
        showSavedCards: false,
        showSaveOption: false,
        features: data('features')
    }
});*/



